@@ -18,29 +18,37 @@ import re
 
 from flake8 import engine
 import pep8
-import testtools
+import pkg_resources
+import six
+import testscenarios
 from testtools import content
 from testtools import matchers
 
-import testscenarios
-
 import hacking
+import hacking.tests
 
 SELFTEST_REGEX = re.compile(r'\b(Okay|[HEW]\d{3}):\s(.*)')
 # Each scenario is (name, dict(lines=.., options=..., code=...))
 file_cases = []
 
 
-class HackingTestCase(testtools.TestCase):
+class HackingTestCase(hacking.tests.TestCase):
 
     scenarios = file_cases
 
     def test_pep8(self):
+
+        # NOTE(jecarey): Add tests marked as off_by_default to enable testing
+        turn_on = set(['H106', 'H203', 'H904', 'H204', 'H205', 'H210'])
+        if self.options.select:
+            turn_on.update(self.options.select)
+        self.options.select = tuple(turn_on)
+
         report = pep8.BaseReport(self.options)
         checker = pep8.Checker(lines=self.lines, options=self.options,
                                report=report)
         checker.check_all()
-        self.addDetail('lines', content.text_content("\n".join(self.lines)))
+        self.addDetail('doctest', content.text_content(self.raw))
         if self.code == 'Okay':
             self.assertThat(
                 len(report.counters),
@@ -50,6 +58,9 @@ class HackingTestCase(testtools.TestCase):
                     [key for key in report.counters
                      if key not in self.options.benchmark_keys]))
         else:
+            self.addDetail('reason',
+                           content.text_content("Failed to trigger rule %s" %
+                                                self.code))
             self.assertIn(self.code, report.counters)
 
 
@@ -59,20 +70,28 @@ def _get_lines(check):
         match = SELFTEST_REGEX.match(line)
         if match is None:
             continue
-        yield match.groups()
+        yield (line, match.groups())
 
 
 def load_tests(loader, tests, pattern):
 
-    flake8_style = engine.get_style_guide(parse_argv=False, ignore='F')
+    flake8_style = engine.get_style_guide(parse_argv=False,
+                                          # Ignore H104 otherwise it's
+                                          # raised on doctests.
+                                          ignore=('F', 'H104'))
     options = flake8_style.options
 
-    for name, check in hacking.core.__dict__.items():
-        if not name.startswith("hacking_"):
+    for entry in pkg_resources.iter_entry_points('flake8.extension'):
+        if not entry.module_name.startswith('hacking.'):
             continue
-        for (lineno, (code, source)) in enumerate(_get_lines(check)):
+        check = entry.load()
+        name = entry.attrs[0]
+        if check.skip_on_py3 and six.PY3:
+            continue
+        for (lineno, (raw, (code, source))) in enumerate(_get_lines(check)):
             lines = [part.replace(r'\t', '\t') + '\n'
                      for part in source.split(r'\n')]
-            file_cases.append(("%s-line-%s" % (name, lineno),
-                              dict(lines=lines, options=options, code=code)))
+            file_cases.append(("%s-%s-line-%s" % (entry.name, name, lineno),
+                              dict(lines=lines, raw=raw, options=options,
+                                   code=code)))
     return testscenarios.load_tests_apply_scenarios(loader, tests, pattern)
